@@ -46,50 +46,42 @@
 */
 #include <locale>
 #ifndef gmic_version
-#define gmic_version 1571
+#define gmic_version 1601
 
 // Define environment variables.
-#ifndef gmic_split_compilation
-#define gmic_float
-#endif // #ifndef gmic_split_compilation
 #ifndef gmic_is_beta
 #define gmic_is_beta 0
 #endif // #ifndef gmic_is_beta
 #ifndef cimg_verbosity
 #define cimg_verbosity 1
 #endif // #ifndef cimg_verbosity
-#if defined(cimg_build)
-#define cimg_plugin "examples/gmic.cpp"
-#define cimg_include_file "../CImg.h"
-#elif defined(gmic_build) // #if defined(cimg_build)
-#define cimg_plugin "gmic.cpp"
-#define cimg_include_file "./CImg.h"
-#endif // #if defined(cimg_build)
 
-#ifdef cimg_include_file
-#include cimg_include_file
+#ifdef gmic_build
+#define cimg_plugin "gmic.cpp"
+#include "./CImg.h"
 #if cimg_OS==2
 #include <process.h>
+#ifdef _MSC_VER
+#pragma comment(linker,"/STACK:16777216")
+#endif // #ifdef _MSC_VER
 #elif cimg_OS==1
 #include <cerrno>
+#include <sys/resource.h>
+#include <signal.h>
+static struct gmic_increase_stack {
+  gmic_increase_stack() {
+    const rlim_t requested_stack_size = 16777216;
+    struct rlimit rl;
+    const int result = getrlimit(RLIMIT_STACK,&rl);
+    if (!result && rl.rlim_cur<requested_stack_size) {
+      rl.rlim_cur = requested_stack_size;
+      setrlimit(RLIMIT_STACK,&rl);
+    }
+  }
+} _gmic_increase_stack;
 #endif // #if cimg_OS==2
 
-// Define some special character codes used for replacement in double quoted strings.
-const char _dollar = 23, _lbrace = 24, _rbrace = 25, _comma = 26, _dquote = 28, _arobace = 29,
-  _newline = 30;
-
-// Replace special characters in a string.
-inline char *gmic_strreplace(char *const str) {
-  for (char *s = str ; *s; ++s) {
-    const char c = *s;
-    if (c<' ')
-      *s = c==_dollar?'$':c==_lbrace?'{':c==_rbrace?'}':c==_comma?',':
-        c==_dquote?'\"':c==_arobace?'@':c;
-  }
-  return str;
-}
-
-#else // #ifdef cimg_include_file
+#else // #ifdef gmic_build
 #include <cstdio>
 #include <cstring>
 
@@ -131,22 +123,38 @@ namespace cimg_library {
   };
 }
 #endif // #ifndef cimg_version
-#endif // #ifdef cimg_include_file
+#endif // #ifdef gmic_build
+
 #define gmic_image cimg_library::CImg
 #define gmic_list cimg_library::CImgList
 #define gmic_display cimg_library::CImgDisplay
 
+// Define some special character codes used for replacement in double quoted strings.
+const char _dollar = 23, _lbrace = 24, _rbrace = 25, _comma = 26, _dquote = 28, _arobace = 29,
+  _newline = 30;
+
+// Replace special characters in a string.
+inline char *gmic_strreplace(char *const str) {
+  for (char *s = str ; *s; ++s) {
+    const char c = *s;
+    if (c<' ')
+      *s = c==_dollar?'$':c==_lbrace?'{':c==_rbrace?'}':c==_comma?',':
+        c==_dquote?'\"':c==_arobace?'@':c;
+  }
+  return str;
+}
+
 // Define the G'MIC exception class.
 //----------------------------------
 struct gmic_exception {
-  gmic_image<char> _command, _message;
+  gmic_image<char> _command_help, _message;
 
   gmic_exception() {}
 
   gmic_exception(const char *const command, const char *const message) {
     if (command) {
-      _command.assign(std::strlen(command)+1,1,1,1);
-      std::strcpy(_command._data,command);
+      _command_help.assign(std::strlen(command)+1,1,1,1);
+      std::strcpy(_command_help._data,command);
     }
     if (message) {
       _message.assign(std::strlen(message)+1,1,1,1);
@@ -157,8 +165,8 @@ struct gmic_exception {
   const char *what() const { // Give the error message returned by the G'MIC interpreter.
     return _message._data?_message._data:"";
   }
-  const char *command() const {
-    return _command._data?_command._data:"";
+  const char *command_help() const {
+    return _command_help._data?_command_help._data:"";
   }
 };
 
@@ -169,31 +177,40 @@ struct gmic {
   // Constructors - Destructors.
   // Use the methods below to create and run the G'MIC interpreter from your C++ source.
 
-  // Destructor.
-  ~gmic();
-
-  // Constructors.
   gmic();
 
-  gmic(const char *const commands_line, const char *const custom_commands=0,
-       const bool include_default_commands=true, float *const p_progress=0, int *const p_cancel=0);
+  gmic(const char *const commands_line,
+       const char *const custom_commands=0,
+       const bool include_default_commands=true,
+       float *const p_progress=0, int *const p_is_cancel=0);
 
   template<typename T>
-  gmic(const int argc, const char *const *const argv, gmic_list<T>& images,
-       gmic_list<char>& images_names,
-       const char *const custom_commands=0, const bool include_default_commands=true,
-       float *const p_progress=0, int *const p_cancel=0);
+  gmic(const char *const commands_line,
+       gmic_list<T>& images, gmic_list<char>& images_names,
+       const char *const custom_commands=0,
+       const bool include_default_commands=true,
+       float *const p_progress=0, int *const p_is_cancel=0);
+
+  ~gmic();
+
+  // Methods to call interpreter on an already constructed gmic instance.
+  gmic& run(const char *const commands_line,
+            float *const p_progress=0, int *const p_is_cancel=0) {
+    gmic_list<float> images;
+    gmic_list<char> images_names;
+    return run(commands_line,images,images_names,
+               p_progress,p_is_cancel);
+  }
 
   template<typename T>
-  gmic(const char *const commands_line, gmic_list<T>& images, gmic_list<char>& images_names,
-       const char *const custom_commands=0, const bool include_default_commands=true,
-       float *const p_progress=0, int *const p_cancel=0);
-
-  // Method to call parser on an already constructed gmic object.
-  template<typename T>
-  gmic& parse(const char *const commands_line,
-              gmic_list<T> &images, gmic_list<char> &images_names) {
-    return parse(commands_line_to_CImgList(commands_line),images,images_names);
+  gmic& run(const char *const commands_line,
+            gmic_list<T> &images, gmic_list<char> &images_names,
+            float *const p_progress=0, int *const p_is_cancel=0) {
+    starting_commands_line = commands_line;
+    is_debug = false;
+    return _run(commands_line_to_CImgList(commands_line),
+                images,images_names,p_progress,p_is_cancel);
+    return *this;
   }
 
   //--------------------------------------------------------------------------------------
@@ -201,21 +218,19 @@ struct gmic {
   // in your own C++ source code. Use them at your own risk.
   //--------------------------------------------------------------------------------------
   template<typename T>
-  void _gmic(const char *const commands_line, gmic_list<T>& images, gmic_list<char>& images_names,
+  void _gmic(const char *const commands_line,
+             gmic_list<T>& images, gmic_list<char>& images_names,
              const char *const custom_commands, const bool include_default_commands,
-             float *const p_progress, int *const p_cancel);
+             float *const p_progress, int *const p_is_cancel);
 
-  gmic& add_commands(const char *const data_commands,
-                     gmic_list<char> (&commands_names)[256],
-                     gmic_list<char> (&commands)[256],
-                     gmic_list<char> (&commands_has_arguments)[256]);
-  gmic& add_commands(std::FILE *const file,
-                     gmic_list<char> (&commands_names)[256],
-                     gmic_list<char> (&commands)[256],
-                     gmic_list<char> (&commands_has_arguments)[256]);
-  gmic_image<char> scope2string(const bool is_last_slash=true) const;
-  gmic_image<char> scope2string(const gmic_image<unsigned int>& scope_selection,
-                                const bool is_last_slash=true) const;
+  gmic& set_variable(const char *const variable_name, const char *const variable_content);
+
+  gmic& add_commands(const char *const data_commands, const char *const commands_file=0);
+  gmic& add_commands(std::FILE *const file, const char *const filename=0);
+
+  gmic_image<char> scope2string() const;
+  gmic_image<char> scope2string(const gmic_image<unsigned int>& scope_selection) const;
+  gmic_image<char> scope2string(const gmic_image<unsigned int>* scope_selection) const;
 
   gmic_image<unsigned int> selection2cimg(const char *const string, const unsigned int indice_max,
                                           const gmic_list<char>& names,
@@ -228,38 +243,27 @@ struct gmic {
 
   gmic_list<char> commands_line_to_CImgList(const char *const commands_line);
 
+  gmic& print(const char *format, ...);
+  gmic& error(const char *format, ...);
+  gmic& debug(const char *format, ...);
+
   template<typename T>
   gmic_image<char> substitute_item(const char *const source,
                                    gmic_list<T>& images, gmic_list<char>& images_names,
-				   unsigned int (&variables_sizes)[256]);
-
-  gmic& print(const char *format, ...);
+                                   gmic_list<T>& parent_images, gmic_list<char>& parent_images_names,
+				   unsigned int variables_sizes[256]);
   template<typename T>
-  gmic& print(const gmic_list<T>& list, const char *format, ...);
-  template<typename T>
-  gmic& print(const gmic_list<T>& list, const gmic_image<unsigned int>& scope_selection,
+  gmic& print(const gmic_list<T>& list, const gmic_image<unsigned int> *const scope_selection,
 	      const char *format, ...);
 
-  gmic& warn(const char *format, ...);
   template<typename T>
-  gmic& warn(const gmic_list<T>& list, const char *format, ...);
-  template<typename T>
-  gmic& warn(const gmic_list<T>& list, const gmic_image<unsigned int>& scope_selection,
+  gmic& warn(const gmic_list<T>& list, const gmic_image<unsigned int> *const scope_selection,
              const char *format, ...);
 
-  gmic& error(const char *format, ...);
   template<typename T>
-  gmic& error(const gmic_list<T>& list, const char *format, ...);
-  template<typename T>
-  gmic& error(const char *const command, const gmic_list<T>& list, const char *format, ...);
-  template<typename T>
-  gmic& error(const gmic_list<T>& list, const gmic_image<unsigned int>& scope_selection,
-	      const char *format, ...);
-  template<typename T>
-  gmic& _arg_error(const gmic_list<T>& list, const char *const command,
-		   const char *const argument);
+  gmic& error(const gmic_list<T>& list, const gmic_image<unsigned int> *const scope_selection,
+	      const char *const command, const char *format, ...);
 
-  gmic& debug(const char *format, ...);
   template<typename T>
   gmic& debug(const gmic_list<T>& list, const char *format, ...);
 
@@ -283,7 +287,8 @@ struct gmic {
   template<typename T>
   gmic& display_objects3d(const gmic_list<T>& images,
                           const gmic_list<char>& images_names,
-                          const gmic_image<unsigned int>& selection);
+                          const gmic_image<unsigned int>& selection,
+                          const gmic_image<unsigned char>& background3d);
   template<typename T>
   gmic_image<T>& check_image(const gmic_list<T>& list, gmic_image<T>& img);
   template<typename T>
@@ -293,78 +298,38 @@ struct gmic {
   gmic& remove_images(gmic_list<T>& images, gmic_list<char>& images_names,
                       const gmic_image<unsigned int>& selection,
                       const unsigned int start, const unsigned int end);
-  template<typename T>
-  gmic& parse(const gmic_list<char>& commands_line,
-	      gmic_list<T> &images, gmic_list<char> &images_names) {
-    unsigned int variables_sizes[256] = { 0 };
-    unsigned int position = 0;
-    std::setlocale(LC_NUMERIC,"C");
-    scope.assign(1U);
-    scope._data[0].assign(2,1,1,1);
-    scope._data[0]._data[0] = '.';
-    scope._data[0]._data[1] = 0;
-    dowhiles.assign(0U);
-    repeatdones.assign(0U);
-    status.assign(0U);
-    is_start = true;
-    is_quit = false;
-    is_return = false;
-    is_default_type = true;
-    *progress = -1;
-    return _parse(commands_line,position,images,images_names,variables_sizes);
-  }
 
   template<typename T>
-  gmic& _parse(const gmic_list<char>& commands_line, unsigned int& position,
-               gmic_list<T> &images, gmic_list<char> &images_names,
-               unsigned int (&variables_sizes)[256]);
-  gmic& _parse_bool(const gmic_list<char>& commands_line, unsigned int& position,
-                    gmic_list<bool>& images, gmic_list<char> &images_names,
-                    unsigned int (&variables_sizes)[256]);
-  gmic& _parse_uchar(const gmic_list<char>& commands_line, unsigned int& position,
-                     gmic_list<unsigned char>& images, gmic_list<char> &images_names,
-                     unsigned int (&variables_sizes)[256]);
-  gmic& _parse_char(const gmic_list<char>& commands_line, unsigned int& position,
-                    gmic_list<char>& images, gmic_list<char> &images_names,
-                    unsigned int (&variables_sizes)[256]);
-  gmic& _parse_ushort(const gmic_list<char>& commands_line, unsigned int& position,
-                      gmic_list<unsigned short>& images, gmic_list<char> &images_names,
-                      unsigned int (&variables_sizes)[256]);
-  gmic& _parse_short(const gmic_list<char>& commands_line, unsigned int& position,
-                     gmic_list<short>& images, gmic_list<char> &images_names,
-                     unsigned int (&variables_sizes)[256]);
-  gmic& _parse_uint(const gmic_list<char>& commands_line, unsigned int& position,
-                    gmic_list<unsigned int>& images, gmic_list<char> &images_names,
-                    unsigned int (&variables_sizes)[256]);
-  gmic& _parse_int(const gmic_list<char>& commands_line, unsigned int& position,
-                   gmic_list<int>& images, gmic_list<char> &images_names,
-                   unsigned int (&variables_sizes)[256]);
-  gmic& _parse_float(const gmic_list<char>& commands_line, unsigned int& position,
-                     gmic_list<float>& images, gmic_list<char> &images_names,
-                     unsigned int (&variables_sizes)[256]);
-  gmic& _parse_double(const gmic_list<char>& commands_line, unsigned int& position,
-                      gmic_list<double>& images, gmic_list<char> &images_names,
-                      unsigned int (&variables_sizes)[256]);
+  gmic& _run(const gmic_list<char>& commands_line,
+             gmic_list<T> &images, gmic_list<char> &images_names,
+             float *const p_progress=0, int *const p_is_cancel=0);
 
-  // Internal environment variables.
+  template<typename T>
+  gmic& _run(const gmic_list<char>& commands_line, unsigned int& position,
+             gmic_list<T>& images, gmic_list<char>&images_names,
+             gmic_list<T>& parent_images, gmic_list<char>& parent_images_names,
+             unsigned int variables_sizes[256],
+             bool *const is_noargs=0);
+
+  // Internal environment variables of the interpreter.
 #if cimg_display!=0
   gmic_display instant_window[10];
 #endif // #if cimg_display!=0
-  gmic_list<char> commands[256], commands_names[256], commands_has_arguments[256],
-    variables[256], variables_names[256], scope;
+  gmic_list<char> *const commands, *const commands_names, *const commands_has_arguments,
+    *const _variables, *const _variables_names, **const variables, **const variables_names,
+    commands_files, scope;
   gmic_list<unsigned int> dowhiles, repeatdones;
-  gmic_image<unsigned char> background3d, light3d;
-  gmic_image<float> pose3d;
+  gmic_image<unsigned char> light3d;
   gmic_image<char> status;
   float focale3d, light3d_x, light3d_y, light3d_z, specular_lightness3d, specular_shininess3d,
     _progress, *progress;
-  bool is_released, is_debug, is_start, is_quit, is_return, is_double3d, is_default_type,
-    check_elif;
+  bool is_released, is_debug, is_start, is_return, is_quit, is_double3d, is_default_type,
+    is_debug_infos, check_elif;
   int verbosity, render3d, renderd3d;
-  volatile int _cancel, *cancel;
-  unsigned int nb_carriages;
+  volatile int _is_cancel, *is_cancel;
+  unsigned int nb_carriages, debug_filename, debug_line, cimg_exception_mode;
   unsigned long reference_time;
-
+  const char *starting_commands_line;
 }; // End of the 'gmic' class.
 
 #endif // #ifndef gmic_version
