@@ -3,32 +3,32 @@
  * @author Sebastien Fourey
  * @date   July 2010
  * @brief Definition of the methods of the class FilterThread.
- *  
+ *
  * This file is part of the ZArt software's source code.
- * 
- * Copyright Sebastien Fourey / GREYC Ensicaen (2010-...) 
- * 
+ *
+ * Copyright Sebastien Fourey / GREYC Ensicaen (2010-...)
+ *
  *                    https://foureys.users.greyc.fr/
- * 
+ *
  * This software is a computer program whose purpose is to demonstrate
  * the possibilities of the GMIC image processing language by offering the
  * choice of several manipulations on a video stream aquired from a webcam. In
  * other words, ZArt is a GUI for G'MIC real-time manipulations on the output
  * of a webcam.
- * 
+ *
  * This software is governed by the CeCILL  license under French law and
- * abiding by the rules of distribution of free software.  You can  use, 
+ * abiding by the rules of distribution of free software.  You can  use,
  * modify and/ or redistribute the software under the terms of the CeCILL
  * license as circulated by CEA, CNRS and INRIA at the following URL
  * "http://www.cecill.info". See also the directory "Licence" which comes
- * with this source code for the full text of the CeCILL licence. 
- * 
+ * with this source code for the full text of the CeCILL licence.
+ *
  * As a counterpart to the access to the source code and  rights to copy,
  * modify and redistribute granted by the license, users are provided only
  * with a limited warranty  and the software's author,  the holder of the
  * economic rights,  and the successive licensors  have only  limited
- * liability. 
- * 
+ * liability.
+ *
  * In this respect, the user's attention is drawn to the risks associated
  * with loading,  using,  modifying and/or developing or reproducing the
  * software by the user in light of its specific status of free software,
@@ -36,10 +36,10 @@
  * therefore means  that it is reserved for developers  and  experienced
  * professionals having in-depth computer knowledge. Users are therefore
  * encouraged to load and test the software's suitability as regards their
- * requirements in conditions enabling the security of their systems and/or 
- * data to be ensured and,  more generally, to use and operate it in the 
- * same conditions as regards security. 
- * 
+ * requirements in conditions enabling the security of their systems and/or
+ * data to be ensured and,  more generally, to use and operate it in the
+ * same conditions as regards security.
+ *
  * The fact that you are presently reading this means that you have had
  * knowledge of the CeCILL license and that you accept its terms.
  */
@@ -56,8 +56,6 @@
 #include <iostream>
 using namespace cimg_library;
 
-const unsigned int FilterThread::_gmic_variables_sizes[27] = { 0 };
-
 FilterThread::FilterThread( WebcamGrabber & webcam,
                            const QString & command,
                            QImage * outputImage,
@@ -125,7 +123,7 @@ FilterThread::run()
 	 _gmic_images[0].assign(_webcam.width(),_webcam.height(),1,3);
 
       ImageConverter::convert( _webcam.image(), _gmic_images[0] );
-      
+
       // Call the G'MIC interpreter.
       try {
 	 QString c( _command );
@@ -138,9 +136,9 @@ FilterThread::run()
 	 c.replace( "@{!,x}", strX )
 	    .replace( "@{!,y}", strY )
 	    .replace( "@{!,b}", strB );
-	 _gmic.parse( c.toAscii().constData(),
-		      _gmic_images,
-		      _gmic_images_names);
+	 _gmic.run( c.toAscii().constData(),
+                    _gmic_images,
+                    _gmic_images_names);
 
 	 switch ( _previewMode ) {
 	 case Full:
